@@ -1,4 +1,4 @@
-import ssl, json, time, sys, queue, atexit, logging
+import ssl, json, time, sys, queue, atexit, logging, threading
 import paho.mqtt.client as paho
 
 
@@ -38,8 +38,19 @@ class MQTTSubscriber:
       time.sleep(sleep)
 
 
+class MQTTMessage:
+  def __init__ (self, topic, payload):
+    self.topic = topic
+    self.payload = payload
+    self.publish_info = None
+  
+  def publish (self, client):
+    self.publish_info = client.publish(self.topic, self.payload)
+
+
+
 class MQTT:
-  def __init__ (self, host, port, client_id, use_ssl=True, transport='websockets'):
+  def __init__ (self, host, port, client_id, use_ssl=False, transport='tcp'):
     self.use_ssl = use_ssl
     self.client_id = client_id
     self.mqtt_host = host
@@ -54,6 +65,8 @@ class MQTT:
     self.client.on_disconnect = self.on_disconnect
     self.client.max_inflight_messages_set(0)
 
+    self.out_messages = queue.Queue()
+
     if self.use_ssl:
         ssl_ctx = ssl.create_default_context()
         ssl_ctx.check_hostname = False
@@ -61,8 +74,32 @@ class MQTT:
         self.client.tls_set_context(ssl_ctx)
         #self.client.tls_insecure_set(True)
 
-    atexit.register(self.wait_for_publish)
+    atexit.register(self.on_exit)
 
+    self.thread = threading.Thread(target=self.main)
+    self.thread.daemon = True
+    self.thread.start()
+
+  def _ensure_connection (self):
+    while 1:
+      try:
+        result = self.client.connect(self.mqtt_host, self.mqtt_port)
+        self.client.loop_start()
+      except:
+        LOG.warning('failed to connect to server')
+        time.sleep(5)
+      else:
+        break
+
+  def main (self):
+    self._ensure_connection()
+    while 1:
+      time.sleep(0.02)
+      if not self.is_connected or self.out_messages.empty():
+        continue
+      msg = self.out_messages.get()
+      msg.publish(self.client)
+            
   def subscribe (self, topic):
     sub = MQTTSubscriber(self.client, topic)
     self.subscriptions.append(sub)
@@ -86,13 +123,12 @@ class MQTT:
       sub.add_message(msg)
     
   def publish (self, topic, msg):
-    info = self.client.publish(topic, json.dumps(msg))
-    return info
-
-  def wait_for_publish (self):
+    self.out_messages.put(
+      MQTTMessage(topic, json.dumps(msg))
+    )
+  
+  def on_exit (self):
     # TODO: save unsent messages to file until confirmed
-    time.sleep(1) 
-
-  def connect (self):
-    result = self.client.connect(self.mqtt_host, self.mqtt_port, keepalive=10)
-    self.client.loop_start()
\ No newline at end of file
+    while not self.out_messages.empty():
+      LOG.info('waiting for pending messages before exit..')
+      time.sleep(1) 
\ No newline at end of file
